#!/usr/bin/env ruby

$KCODE = "EUC"

require 'cgi'

require 'config'
require 'login'
require 'player'
require 'vil'
require 'skill'
require 'util'
require 'catalog'
require 'errormsg'
require 'tsslock'
require 'tss'

class CharsetDump
  def dump(k)
    NAMES.each_index do |i|
      next if i == 0
      print announce(c(JOINING, i, NAMES[i]))
      print %Q(<img src="img/body#{two(i)}#{k}.jpg"> )
      print %Q(<img src="img/face#{two(i)}#{k}.jpg">)
    end
  end
  def CharsetDump.clear_const
    constants.each do |c|
      remove_const(c)
    end
  end
end

class CWolf
  def print_head(title = nil)
    print(HEAD1)

    if (title)
      print "<title>͹ #{title}</title>"
    else
      print "<title>͹</title>"
    end

    print(HEAD2)
  end

  def get_vil_lock(vid)
    v = @vildb.transaction(true) do
      @vildb['root']
    end
    v.catalog_init
    v
  end

  def get_vil(vid)
    v = @vildb['root']
    v.catalog_init
    v
  end

  def get_srvs
    srvs = Hash.new
    srvdb = Store.new('db/srv.db')
    srvdb.transaction(true) do
      srvdb.roots.each do |k|
        srvs[k] = srvdb[k]
      end
    end
    srvs
  end

  def initialize
    @cgi = CGI.new()
    @vid = @cgi['vid'].to_i
    if (@vid >= 100)
      @vildb = Store.new("db/vil/#{@vid}.db")
    end
  end

  def handle_mkvil
    raise ErrorMsg.new('󤷤Ʋ') if !@login.login
    raise ErrorMsg.new('¼̾ꤷƲ') if (@cgi['name'].to_s == '')

    gert = (@cgi['gert'] == 'on')
    guts = (@cgi['guts'] == 'on')
    anon = (@cgi['anon'] == 'on')
    ded = (@cgi['ded'] == 'on')
#    commit = (@cgi['commit'] == 'on')
    commit = true
#    moe = (@cgi['moe'] == 'on')
    char = @cgi['char'].to_s
    char = nil if (char == '' || char == 'default')
    comment = @cgi['comment']

    preview = (@cgi['preview'] == 'on')
    guts2 = (@cgi['guts2'] == 'on')
    action = (@cgi['action'] == 'on')
    firstnight = (@cgi['firstnight'] == 'on')

    period = @cgi['time'].to_i * @cgi['period'].to_i
    night_period = @cgi['night_time'].to_i * @cgi['night_period'].to_i

    level = @cgi['level'].to_i

    nums = Array.new
    for i in 0..Skill.skills.size
      nums.push(@cgi["num_#{i}"].to_i)
    end

    all = 0
    nums.each do |n|
      all += n
    end
    max = CHAR_MAX_NUM
    raise ErrorMsg.new("#{max}ͰʾϥǤ") if (all > max)

    raise ErrorMsg.new("ߡ¼ͤǤ") if (gert && nums[0] == 0)

    lock = TSSLock.new(COUNTRY_NAME)
    raise ErrorMsg.new('ʤ󤫥åǤƤޤ󡣤ΥФŪƤΤǤС뤵Ƥ붲줬ޤǤʤʤ餳Υ顼ϰŪʤΤǤ') if (!lock.lock('vil'))

    begin
      vldb = Store.new('db/vil.db')
      vid = vldb.transaction do
        vid = vldb['recent_vid'].to_i + 1
        vldb['recent_vid'] = vid

        vil = Hash.new
        vil['name'] = @cgi['name']
        vil['vid'] = vid
        vil['period'] = period
        vil['night_period'] = night_period
        vil['state'] = 0
        vil['country'] = COUNTRY_NAME
        vil['level'] = level
        vldb['root'].push(vil)

        vid
      end

      @vildb = Store.new("db/vil/#{vid}.db")
      vil = @vildb.transaction do
        vil = Vil.new(@cgi['name'], vid, nums, comment,
                      @login.userid, @login.nick,
                      gert, commit, period, guts, char, night_period, anon, ded)
        vil.firstnight = firstnight
        vil.action = action
        vil.guts2 = guts2
        vil.preview = preview
        @vildb['root'] = vil

        if (gert)
          twop = (vil.moe) ? vil.moe : nil
          player = Player.new(1, MASTER, MASTER_NICK, 0, twop)
          vil.add_player(player)
        end

        vil
      end

      vid = vil.vid

      File.open("db/vil/#{vid}_0.html", 'a') do |of|
        of.flock(File::LOCK_EX)
        of.print(announce(OPENING))
      end

      if (gert)
        player = vil.player_p(1)

        s = announce(c(JOINING, 1, player.name))
        s += message(vil.say('say', nil, player, GERT_ENTRY))
        vil.addlog(s)
      end

      http_broadcast('db/vil.db')
    ensure
      lock.unlock('vil')
    end
  end

  def handle_edvil
    raise ErrorMsg.new('󤷤Ʋ') if !@login.login
    raise ErrorMsg.new('¼̾ꤷƲ') if (@cgi['name'].to_s == '')

    gert = (@cgi['gert'] == 'on')
    guts = (@cgi['guts'] == 'on')
    anon = (@cgi['anon'] == 'on')
    ded = (@cgi['ded'] == 'on')
#    commit = (@cgi['commit'] == 'on')
    commit = true
    comment = @cgi['comment']

    preview = (@cgi['preview'] == 'on')
    guts2 = (@cgi['guts2'] == 'on')
    action = (@cgi['action'] == 'on')
    firstnight = (@cgi['firstnight'] == 'on')

    period = @cgi['time'].to_i * @cgi['period'].to_i
    night_period = @cgi['night_time'].to_i * @cgi['night_period'].to_i

    nums = Array.new
    for i in 0..Skill.skills.size
      nums.push(@cgi["num_#{i}"].to_i)
    end

    all = 0
    nums.each do |n|
      all += n
    end
    max = CHAR_MAX_NUM
    raise ErrorMsg.new("#{max}ͰʾϥǤ") if (all > max)

    raise ErrorMsg.new("#{GERT}¼ͤǤ") if (gert && nums[0] == 0)

    @vildb.transaction do
      vil = get_vil(@vid)

      if (vil.userid != @login.userid && !@login.master?)
        raise ErrorMsg.new('ʤ¼ԤǤϤޤ')
      end

      if (vil.state > 1)
        raise ErrorMsg.new('Ϥ¼ԽǤޤ')
      end

      gertoff = !gert && vil.gert
      gerton = gert && !vil.gert

      if (gerton && !vil.left_pids.find { |p| p == 1 })
        raise ErrorMsg.new("#{GERT}äƤޤ")
      end

      max_players = 0
      nums.each do |n|
        max_players += n
      end
      if (vil.players.size > max_players)
        raise ErrorMsg.new('˻äƤͿ¼͸ۤƤޤ')
      end

#      vil = get_vil(@vid)

      if (vil.name != @cgi['name'] ||
          vil.period != period || vil.night_period != night_period)
        lock = TSSLock.new(COUNTRY_NAME)
        raise ErrorMsg.new('ʤ󤫥åǤƤޤ') if (!lock.lock('vil'))

        begin
          vldb = Store.new('db/vil.db')
          vldb.transaction do
            v = vldb['root'].find() do |v|
              v['vid'] == @vid
            end
            v['name'] = @cgi['name']
            v['period'] = period
            v['night_period'] = night_period
          end

          http_broadcast('db/vil.db')

          lock.unlock('vil')
        ensure
          lock.unlock('vil')
        end
      end

      vil.edit(@cgi['name'], nums, comment, gert, guts, commit, period, night_period, anon, ded)
      vil.firstnight = firstnight
      vil.action = action
      vil.guts2 = guts2
      vil.preview = preview

      if (gerton)
        twop = (vil.moe) ? vil.moe : nil
        player = Player.new(1, MASTER, MASTER_NICK, 0, twop)
        vil.add_player(player)

        num = vil.players.size
        s = announce(c(JOINING, num, player.name))
        s += message(vil.say('say', nil, player, GERT_ENTRY))
        vil.addlog(s)
      elsif (gertoff)
        s = announce(c(PARTING, vil.players[MASTER].name))
        vil.addlog(s)

        vil.players.delete(MASTER)
      end
    end
  end

  def handle_message
    raise ErrorMsg.new("¼IDǤ") if (@vid == 0)

    pid = @cgi['pid']
    raise ErrorMsg.new("ʥ饯Ǥ") if (!pid)
    pid = pid.to_i

    msg = @cgi['message']

    type =
      if (@cgi['think'] == 'on')
        'think'
      elsif (@cgi['whisper'] == 'on')
        'whisper'
      elsif (@cgi['groan'] == 'on')
        'groan'
      elsif (@cgi['action'] == 'on')
        if (@cgi['std_action'] == 'on')
          if @cgi['action_target'] == ''
            raise ErrorMsg.new('оݤꤵƤޤ')
          end
          msg = @cgi['std_message']
        end
        msg = @cgi['action_target'] + msg
        'action'
      else
        'say'
      end

    return if (!msg || msg == '')

    msg = CGI::escapeHTML(msg)
    msg.gsub!(/\r\n/, '<br>')
    msg.gsub!(/[\r\n]/, '<br>')

    @vildb.transaction do
      vil = get_vil(@vid)

      raise ErrorMsg.new('λƤޤ') if (vil.state == 4)

      if (pid != 0)
        player = vil.player(@login)
        raise ErrorMsg.new("ʥ饯Ǥ(ʤϵ᤭ξ硢ǤäƥɤƤȯƲ)") if (player.pid != pid)

        if (!player.can_whisper && type == 'whisper')
          raise ErrorMsg.new('ʤϤ䤱ޤ')
        end

        if (vil.state == 2 && type == 'say' && /&gt;&gt;(\d+:)?\*\d+/ =~ msg)
          raise ErrorMsg.new('㤤')
        end

        return if (player.prev_msg == msg)
        player.prev_msg = msg

        if (player.dead != 0 && type == 'say')
          type = 'groan'
        end
        if (player.dead == 0 && type == 'groan')
          type = 'say'
        end

        if (vil.night && type == 'say')
          type = 'think'
        end

        if type != 'action'
          vil.say_cnt[type] = vil.say_cnt[type] + 1
          cnt = vil.say_cnt[type]
        end

        if (type == 'say')
          player.say(vil.date, msg.size)
        end

        msg = "<div class=\"loud\">#{msg}</div>" if (@cgi['loud'] == 'on')

        if type == 'action'
          s = vil.say_action(type, player, msg)
          vil.addlog(message(s)) if s != ''
        else
          s = vil.say(type, cnt, player, msg)
          vil.addlog(message(s))
        end
      else
        return if (vil.date != 0 && type == 'say')

        vil.say_cnt[type] = vil.say_cnt[type] + 1
        cnt = vil.say_cnt[type]

        msg = "<div class=\"loud\">#{msg}</div>" if (@cgi['loud'] == 'on')

        s = vil.say_impl(type, cnt, 0, "˵Ѽ #{@login.nick}", 'face00', msg)
        vil.addlog(message(s))
      end
    end
  end

  def handle_entry
    raise ErrorMsg.new("¼IDǤ") if (@vid == 0)

    pid = @cgi['pid']
    raise ErrorMsg.new("ʥ饯Ǥ") if (!pid)
    pid = pid.to_i

    msg = @cgi['message']
    raise ErrorMsg.new("ȯǤ") if (!msg || msg == '')

    skill =
      if (@cgi['skill'] != '')
        @cgi['skill'].to_i
      else
        -1
      end

    vil, player = @vildb.transaction do
      vil = get_vil(@vid)
      if (vil.players.key?(@login.userid))
        raise ErrorMsg.new("ȥ꡼ѤߤǤ")
      elsif (vil.state != 0 && vil.state != 1)
        raise ErrorMsg.new('˳ϤƤޤ')
      elsif (vil.max_players == vil.players.size)
        raise ErrorMsg.new("ãƤޤ")
      elsif (vil.players.find { |p| p[1].pid == pid })
        raise ErrorMsg.new("˻ѤƤ륭饯Ǥ")
      end

#      twop =
#        if (@cgi['twop'] == 'yes')
#          '_2'
#        else
#          nil
#        end
#      if (vil.moe)
#        twop = '_moe'
#      end
      twop = (vil.moe) ? vil.moe : nil

      player = Player.new(pid, @login.userid, @login.nick, skill, twop)
      player.say(0, msg.size)
      vil.add_player(player)

      vil.say_cnt['say'] = vil.say_cnt['say'] + 1

      num = vil.players.size
      cnt = vil.say_cnt['say']

      msg = CGI::escapeHTML(msg)
      msg.gsub!(/\r\n/, '<br>')
      msg.gsub!(/[\r\n]/, '<br>')

      s = announce(c(JOINING, num, player.name))
      if (defined?(FIRST_BODY) && FIRST_BODY)
        face = "body#{player.color}"
        s += message(vil.say("say", cnt, player, msg, face))
      else
        s += message(vil.say("say", cnt, player, msg))
      end
      vil.addlog(s)
    end
  end

  def handle_exit
    raise ErrorMsg.new("¼IDǤ") if (@vid == 0)

    pid = @cgi['pid']
    raise ErrorMsg.new("ʥ饯Ǥ") if (!pid)
    pid = pid.to_i

    @vildb.transaction do
      vil = get_vil(@vid)
      if (!vil.players.key?(@login.userid))
        raise ErrorMsg.new("ȥ꡼Ƥޤ")
      elsif (vil.date == 1)
        raise ErrorMsg.new("˳ϤƤޤ")
      end

      s = announce(c(PARTING, vil.players[@login.userid].name))
      vil.addlog(s)

      vil.players.delete(@login.userid)
    end
  end

  def handle_commit
    raise ErrorMsg.new("¼IDǤ") if (@vid == 0)

    pid = @cgi['pid']
    raise ErrorMsg.new("ʥ饯Ǥ") if (!pid)
    pid = pid.to_i

    @vildb.transaction do
      vil = get_vil(@vid)
      if (!vil.players.key?(@login.userid))
        raise ErrorMsg.new("ȥ꡼Ƥޤ")
      elsif (!vil.commit)
        raise ErrorMsg.new("ߥåȤǤ¼ǤϤޤ")
      elsif (vil.state != 2)
        raise ErrorMsg.new("ʹǤϤޤ")
      end

      vil.players[@login.userid].commit = !vil.players[@login.userid].commit

      if (!vil.pids.find { |p| !p.commit })
        vil.update(nil)
      end
    end
  end

  def handle_update
    move = @vildb.transaction do
      vil = get_vil(@vid)

      vil.update(@cgi['nocnt'] == '1')

      (vil.state == 4)
    end

    if (move)
      print "Status Code: 302 Moved Temporary\n"
      print "Location: log_#{@vid}_#{0}.html\n\n"
      exit(0)
    end
  end

  def handle_vote
    raise ErrorMsg.new('ɼ褬åȤƤޤ') if (!@cgi['pid'])
    @vildb.transaction do
      vil = get_vil(@vid)
      vil.player(@login).vote = @cgi['pid'].to_i
    end
  end

  def handle_skill
    @vildb.transaction do
      vil = get_vil(@vid)
      player = vil.player(@login)
      player.skill.action(@cgi, vil, player)
    end
  end

  def handle_end_comment
    @vildb.transaction do
      vil = get_vil(@vid)
      vil.end_comment = @cgi['end_comment']
    end
  end

  def handle_recnt(vid)
    vildb = Store.new("db/vil/#{vid}.db")
    vildb.transaction do
      vil = vildb['root']
      vil.recnt
    end
  end

  def handle_cmd
    cmd = @cgi['cmd']

    if (cmd == 'mkvil')
      handle_mkvil
    elsif (cmd == 'edvil')
      handle_edvil
    elsif (cmd == 'entry')
      handle_entry
    elsif (cmd == 'msg')
      handle_message
    elsif (cmd == 'update')
      handle_update
    elsif (cmd == 'vote')
      handle_vote
    elsif (cmd == 'skill')
      handle_skill
    elsif (cmd == 'exit')
      handle_exit
    elsif (cmd == 'commit')
      handle_commit
    elsif (cmd == 'end_comment')
      handle_end_comment
    end
  end

  def handle_index
    if (@login.master?)
      print '<h2>master</h2><p>'
      userdb = Store.new('db/user.db')
      userdb.transaction(true) do
        print userdb.roots.sort.collect {|i| "<a href=\"matome.cgi?#{CGI.escape(i)}\">#{userdb[i]['nick']}</a>"}.join(', ')
      end
      print '</p>'
    end

    print "<h2>͹ - #{COUNTRY_NAME}</h2>"
    print INDEX_HEAD

    print "<h2>ʬа</h2>"

    srvdb = Store.new('db/srv.db')
    srvdb.transaction(true) do
      print "<p>"
      l = srvdb.roots.sort.collect do |n|
        s = srvdb[n]
        %Q(<a href="http://#{s['serv']}#{s['path']}">#{n}</a>)
      end.join(' / ')
      print l
    end

    print "<h2>ӤޤȤ</h2>"
    print "<p><a href=\"http://#{INIT_SERV}#{INIT_PATH}ranking.html\">ӤޤȤ</a></p>"
    if (@login.login)
      id = @login.userid
      print "<p><a href=\"matome.cgi?#{CGI.escape(id)}\">#{@login.nick}</a></p>"
    end

    print "<h2>¼Υꥹ</h2>"

    srvdb.transaction(true) do
      vldb = Store.new('db/vil.db')
      vldb.transaction(true) do
        break if (!vldb['root'])

        vils = vldb['root'].select { |v| v['state'] != 4 }.reverse
        [3, 2, 1, 0].each do |level|
          print(VILS_HEAD[level])
          vils.select { |v| v['level'].to_i == level }.each do |v|
            s = srvdb[v['country']]
            next if (s == nil)
            print "<tr><td><a href=\"http://#{s['serv']}#{s['path']}index.cgi?vid=#{v['vid']}#bottom\">#{v['vid']} #{v['name']}</a></td><td>#{Vil.update_str(v['period'], v['night_period'])}</td><td>#{STATES[v['state']]}</td><td>#{v['country']}</td></tr>\n"
          end
          print "</table>"
        end
      end
    end

    print(INDEX_FOOT)
#    erbrun('skel/index_foot.html')
  end

  def handle_chars
    print "<h2> @#{COUNTRY_NAME}</h2>\n"
    CHARSETS.each do |k, v|
      print "<h3>#{v} 饻å</h3>\n"
      CharsetDump.clear_const
      cd = CharsetDump.new
      File.open("catalog_#{k}.rb") do |ifile|
        CharsetDump.class_eval(ifile.read)
      end
      k = (k == 'default') ? nil : "_#{k}"
      cd.dump(k)
    end
  end

  def handle_changes
    print "<h2></h2>\n"
    changes = Array.new
    changes.concat(TSS_UPDATE)
    changes.concat(ORIGINAL_UPDATE)
    changes.sort! do |x, y|
      y[0] <=> x[0]
    end
    changes.each do |d|
      desc = ORIGINAL_UPDATE.include?(d) ? " (#{COUNTRY_NAME}ȼѹ)" : ''
      print "<h3>#{d.shift}#{desc}</h3><ul>"
      d.each do |c|
        print "<li>#{c.sub(/\([^)]*\)$/, '')}"
      end
      print "</ul>\n"
    end
  end

  def print_logs(r, mode, cr_dist, cr_country)
    srvs = get_srvs

    r.each do |v|
      dist = v['dist']
      country = v['country']

      next if (cr_dist != '' && cr_dist != dist)
      next if (cr_country != '' && cr_country != country)

      if (v['is_counted'])
        print "#{v['winner']}"
      else
        print "#{v['winner'] || ''}"
      end

      print "<a href=\"?cmd=log&mode=#{mode}&dist=#{CGI.escape(dist)}\">#{dist}</a>"
      print " #{v['date']}"
      if (v['mvp'] =~ / \(([^)]+)\)/)
        print " #{$1}"
      end

      print %Q( @<a href="?cmd=log&mode=#{mode}&country=#{CGI.escape(country)}">#{country}</a>)

      print "<br>"

      s = srvs[country]
      l = "http://#{s['serv']}#{s['path']}log_#{v['vid']}_info.html"
      print "<a href=\"#{l}\">#{v['vid']} #{v['name']}</a>"

      if (@login.admin? && v['is_counted'] && country == COUNTRY_NAME)
        print %Q( <input type="radio" name="recnt" value="#{v['vid']}">)
      end

      print "<br><br>\n"
    end
  end

  def handle_log
    print "<h2>λ¼εϿ</h2><br>\n"

    recnt = @cgi['recnt'].to_i
    if (recnt > 0 && @login.admin?)
      handle_recnt(recnt)
    end

    vldb = Store.new('db/vil.db')
    vldb.transaction(true) do
      break if (!vldb['root'])

      cr_dist = @cgi['dist']
      cr_country = @cgi['country']

      mode = @cgi['mode'].to_i

      rev = vldb['root'].reverse.select do |v|
        v['state'] == 4 &&
        (cr_dist == '' || cr_dist == v['dist']) &&
        (cr_country == '' || cr_country == v['country'])
      end

      vwins = 0
      wwins = 0
      ywins = 0
      stats = Hash.new
      rev.each do |v|
        next if (!v['is_counted'])

        if (!stats.key?(v['dist']))
          volks = v['dist'].size / 2
          volks -= 1 if (v['dist'] =~ /^/)
          volks -= 2 if (v['dist'] =~ /\/$/)
          stats[v['dist']] = [0, 0, 0, volks]
        end

        if (v['winner'] == '¼')
          vwins += 1
          stats[v['dist']][0] += 1
        elsif (v['winner'] == 'ϵ')
          wwins += 1
          stats[v['dist']][1] += 1
        elsif (v['winner'] == '')
          ywins += 1
          stats[v['dist']][2] += 1
        end
      end

      if mode == 0 || mode == 2
        print "о¼Ǥϡ¼#{vwins}, ϵ#{wwins}"
        print ", #{ywins}" if ywins > 0
        print "<br><br>"

        if (@login.admin?)
          print %Q(<form action="." method="GET"><input type="hidden" name="cmd" value="log">)
        end
      end

      if mode == 0 || mode == 2
        print_logs(rev.select {|v| v['is_counted']}, mode, cr_dist, cr_country)
      end

      if mode == 1 || mode == 2
        print_logs(rev.select {|v| !v['is_counted']}, mode, cr_dist, cr_country)
      end

      if mode == 0 || mode == 2
        if (@login.admin?)
          print %Q(å¼<input type="submit" value="¼"></form><br>)
        end
      end

      if mode == 2 || mode == 3
        print "<h2>ͿʬȤξԷ</h2>"
        prev_num = -1
        stats.sort do |x, y|
          s = x[1][3] <=> y[1][3]
          s = 1 if (s == 0 && x[0] =~ /^/ && y[0] !~ /^/)
          s = -1 if (s == 0 && x[0] !~ /^/ && y[0] =~ /^/)
          s = x[0] <=> y[0] if (s == 0)
          s
        end.each do |s|
          if (prev_num != s[1][3])
            prev_num = s[1][3]
            print "<h3>#{prev_num}</h3>"
          end
          print %Q(<a href="?cmd=log&mode=#{mode}&dist=#{CGI.escape(s[0])}">#{s[0]}</a>: ¼#{s[1][0]}, ϵ#{s[1][1]})
          if (s[0] =~ //)
            print ", #{s[1][2]}"
          end
          print "<br>"
        end

        print "<br>"
        print SHIKOKU_REF
      end

      print "<br>"
      print '<a href="?cmd=log&mode=0">̾¼</a><br>' if mode != 0
      print '<a href="?cmd=log&mode=1">¼ˤʤä¼</a><br>' if mode != 1
      print '<a href="?cmd=log&mode=3">ʬȤξԽ</a><br>' if mode != 3
    end

    print(FOOT)
#    erbrun('skel/foot.html')
  end

  def handle_vid
    @vil = get_vil_lock(@vid)

    if (@vil.state < 3 && @vil.update_time && @vil.update_time < Time.now.to_i)
      @vildb.transaction do
        @vil = get_vil(@vid)
        if (@vil.update_time < Time.now.to_i)
          @vil.update(nil)
        end
      end
    end

    if (@vil.state == 4)
      raise ErrorMsg.new("λƤޤ<a href=\"log_#{@vid}_info.html\">ؤɤ</a>") 
    end

    date =
      if (@cgi.key?('date'))
        @cgi['date'].to_i
      else
        @vil.date
      end

#    erbrun('skel/head.html') 
    print_head("#{@vil.vid} #{@vil.name}")
    @login.form(@vid)

    print "#{@vil.vid} #{@vil.name}<strong>#{@vil.update_str}</strong>\n"

    print "<p>"
    for i in -1..@vil.date
      datestr =
        if (i == 0)
          "ץ"
        elsif (i == -1)
          ""
        elsif (@vil.state == 3 && i == @vil.date)
          "ԥ"
        else
          if (@vil.night && i == @vil.date)
            "#{i}"
          else
            "#{i}"
          end
        end
      if (i == date)
        print "<span class=\"time\">#{datestr}</span> "
      else
        print "<a href=\"?vid=#{@vid}&amp;date=#{i}\">#{datestr}</a> "
      end
    end
    print "</p>\n"

    if (date == -1)
      erbrun('skel/info.html')
      print(FOOT)
      return
    end

    @player = @vil.player(@login)
    print %Q(<div id="discuss">\n)
    File.open("db/vil/#{@vid}_#{date}.html") do |ifile|
      ifile.each do |line|
        if (line =~ /^<div class="message"><div id="([a-z]+)(\d+)p?">/ ||
            line =~ /<div class="message"><!--([a-z]+)(\d+)-->/)
          if (@vil.state != 3)
            pid = $2.to_i
            if ($1 == 'think' || $1 == 'extra')
              next if (@player == nil)
              if (pid != @player.pid)
                if (!($1 == 'think' && @vil.state == 2 &&
                      @player.sid == 14 &&
                      (tp = @vil.player_p(pid)) && tp.sid == 14 &&
                      @player.dead == 0 && tp.dead == 0))
                  next
                end
              end
            elsif ($1 == 'whisper')
              if (@player == nil || !@player.can_whisper)
                next
              end
            elsif ($1 == 'groan')
              if (@player == nil || @player.dead == 0)
                next
              end
            end
          end
        end
        print line
      end
    end

    if (@vil.date == date && @vil.update_time && @vil.state != 3)
      t = Time.at(@vil.update_time)
      dt = Time.at(t - Time.now)
      hour = dt.to_i / 3600
      s = c(SHOW_UPDATE, t.month, t.day, timestr(t), hour, dt.min, dt.sec)
      s += SHOW_UPDATE_SCRIPT
      if (@vil.commit)
        s += c(SHOW_COMMITS, @vil.pids.select { |p| p.commit }.size)
      end
      print(message('<div class="order">'+s+'</div>'))
    end
    print %Q(</div>\n)

    if (@login.login && date == @vil.date)
      spectator = false
      if (@player)
        if (@player.dead == 0)
          erbrun('skel/msg.html')
        else
          erbrun('skel/rip.html')
        end
      elsif (@vil.state == 0 ||
             (@vil.state == 1 && @vil.max_players > @vil.players.size))
        erbrun('skel/entry.html')
      else
        spectator = true
      end

      if (@vil.date == 0)
        spectator = true
      end

      if (spectator)
        erbrun('skel/spectator.html')
      end
    end

    need_force = false
    if ((@vil.state == 0 || @vil.state == 1 || @vil.state == 3) &&
        @login.login && @login.userid == @vil.userid)
      need_force = true
    end

    if (@login.master?)
      need_force = true
    end

    if (need_force)
      erbrun('skel/force.html')
    end

    if (@login.admin? && @vil.state == 3)
      print %Q(
<form class="force_form" action="." method="post" onSubmit="return confirm('¼¼ޤ');">
<input type="hidden" name="cmd" value="update">
<input type="hidden" name="vid" value="#{@vil.vid}">
<input type="hidden" name="nocnt" value="1">
<input type="submit" value="¼¼">
</form>
      )
    end

    if (date != @vil.date)
      print("<a href=\"?vid=#{@vid}&date=#{date+1}\"></a>\n")
    end

    @vil.print_footer(STDOUT, date, @login.filter)
  end

  def run
    headered = false
    head = "Content-Type: text/html; charset=EUC-JP\r\n"
#    head += "Cache-control: no-cache\r\n"
#    head += "Pragma: no-cache\r\n"

    begin
      @login = Login.new(@cgi)

      if (@login.cookie)
        head += "Set-Cookie: #{@login.cookie}\r\n"
      end

      if (!@login.cookie && ENV['REQUEST_METHOD'] == 'POST')
        handle_cmd

        print "Status Code: 302 Moved Temporary\n"
        if (@vid != 0)
          print "Location: ?vid=#{@vid}#bottom\n\n"
        else
          print "Location: .\n\n"
        end
        return
      end

      print head + "\r\n"
      headered = true

      if (@cgi['cmd'] == 'log')
        print_head
        @login.form
        handle_log
        return
      elsif (@cgi['cmd'] == 'doc')
        print_head
        @login.form
        print(DOCUMENT)
        print(FOOT)
        return
      elsif (@cgi['cmd'] == 'changes')
        print_head
        @login.form
        handle_changes
        print(FOOT)
        return
      elsif (@cgi['cmd'] == 'chars')
        print_head
        @login.form
        handle_chars
        print(FOOT)
        return
      elsif (@cgi['cmd'] == 'mkvil')
        print_head
        @login.form
        erbrun('skel/mkvil.html')
        print(FOOT)
        return
      elsif (@cgi['cmd'] == 'edvil')
        print_head
        @login.form
        @vil = get_vil_lock(@vid)
        erbrun('skel/edvil.html')
        print(FOOT)
        return
      end

      if (@vid != 0)
        handle_vid
      else
        print_head
#        erbrun('skel/head.html') 
        @login.form(@vid)
        handle_index
      end
    rescue ErrorMsg
      if (!headered)
        print head + "\r\n"
      end
      print $!
    rescue
      if (!headered)
        print head + "\r\n"
      end
      print "<pre>\n"
      print CGI.escapeHTML("#{$!.to_s}\n")
      print CGI.escapeHTML("#{$!.backtrace.join("\n")}\n")
      print "</pre>\n"
    end
  end
end

CWolf.new.run
