require 'login'
require 'util'
require 'catalog'
require 'user_record'
require 'tss'

class Vil
  attr_reader :name, :date, :vid, :state, :max_players, :nums, :players
  attr_reader :comment, :userid, :nick, :gert, :commit, :period, :update_time
  attr_reader :guts, :moe, :anon, :ded
  attr_accessor :say_cnt
  attr_reader :winner, :mvp
  attr_accessor :end_comment
  attr_reader :night, :night_period
  attr_reader :counted

  attr_accessor :preview, :guts2, :action, :firstnight

  def catalog_init
    if !defined?(GERT)
      charset = moe ? @moe : 'default'
      require "catalog_#{charset}"
    end
  end

  def initialize(name, vid, nums, comment, userid, nick,
                 gert, commit, period, guts, moe, night_period, anon, ded)
    @name = name
    @vid = vid
    @nums = nums
    @date = 0
    @state = 0

    @comment = comment
    @userid = userid
    @nick = nick
    @gert = gert
    @commit = commit
    @period = period
    @guts = guts
    @moe = moe
    @night_period = (night_period == 0) ? nil : night_period
    @anon = anon
    @ded = ded

    @max_players = 0
    @nums.each do |n|
      @max_players += n
    end

    @players = Hash.new

    @say_cnt = Hash.new
    @say_cnt['say'] = 0
    @say_cnt['think'] = 0
    @say_cnt['groan'] = 0
    @say_cnt['whisper'] = 0

    catalog_init
  end

  def edit(name, nums, comment, gert, guts, commit, period, night_period,
           anon, ded)
    @name = name
    @nums = nums
    @comment = comment
    @gert = gert
    @guts = guts
    @commit = commit
    @period = period
    @night_period = (night_period == 0) ? nil : night_period
    @anon = anon
    @ded = ded

    @max_players = 0
    @nums.each do |n|
      @max_players += n
    end
  end

  def update_str
    Vil.update_str(@period, @night_period)
  end

  def Vil.update_str(period, night_period = nil)
    if (period % 60 == 0)
      str = "#{period / 60}ֹ"
    else
      str = "#{period}ʬ"
    end
    if (night_period && night_period != 0)
      if (night_period % 60 == 0)
        str << "+#{night_period / 60}"
      else
        str << "+#{night_period}ʬ"
      end
    end
    str
  end

  def player(login)
    if (!login.login)
      return nil
    end
    return players[login.userid]
  end

  def player_p(pid)
    # ad-hoc festival
    p = players.find do |p|
      p[1].pid == pid && p[1].dead == 0
    end
    return p[1] if p
    p = players.find do |p|
      p[1].pid == pid
    end
    (p) ? p[1] : nil
  end

  def left_pids
    a = Array.new
    names = NAMES
    for i in 1..names.size-1
      a.push(i) if (!@players.find { |p| p[1].pid == i })
    end
    a
  end

  def all_pids
    @players.values.sort do |x, y|
      x.pid <=> y.pid
    end
  end

  def pids
    @players.values.select do |v|
      v.dead == 0
    end.sort do |x, y|
      x.pid <=> y.pid
    end
  end

  def pids_s(sid)
    pids.select do |p|
      p.sid == sid
    end
  end

  def wolves
    pids_s(1) + pids_s(16) + pids_s(17)
  end

  def add_player(player)
    @players[player.userid] = player
    change_state_sync(1) if (@players.size == @max_players)
  end

  def change_state_sync(st)
    lock = TSSLock.new(COUNTRY_NAME)
    raise ErrorMsg.new('ʤ󤫥åǤƤޤ') if (!lock.lock('vil'))

    begin
      @state = st

      vldb = Store.new('db/vil.db')
      vldb.transaction do
        v = vldb['root'].find() do |v|
          v['vid'] == @vid
        end
        v['state'] = st
      end

      http_broadcast('db/vil.db')

      lock.unlock('vil')
    ensure
      lock.unlock('vil')
    end
  end

  def addresmsg(p, t)
    res = if (t.sid == 1 || t.sid == 16 || t.sid == 17)
            "<strong>ϵ</strong>"
          else
            "ʹ"
          end
    p.msg = p.msg.to_s + c(SEER_RESULT, t.name, res) + '<br>'
  end

  def do_end(nocnt)
    lock = TSSLock.new(COUNTRY_NAME)
    raise ErrorMsg.new('ʤ󤫥åǤƤޤ') if (!lock.lock('vil'))

    begin
      votes = Hash.new
      max = 0
      pids.each do |p|
        next if (@gert && p.pid == 1)
        t = player_p(p.vote)
        if (t)
          votes[t] = (votes[t] || 0) + 1
          max = votes[t] if (votes[t] > max)
        end
      end
      picked = votes.select { |k, v| v == max }
      if (picked.size > 1)
        @mvp = "Ʊ #{max}ɼ"
      elsif (picked.size != 0)
        m = picked[0][0]
        @mvp = "#{m.name} (#{m.nick}) #{max}ɼ"
      end

      unlinks = Array.new
      for date in -1..@date
        ds = if (date == -1) then 'info' else date.to_s end
        fname = "log_#{@vid}_#{ds}.html"
        File.open(fname, 'w') do |of|
          of.print(HEAD.sub(/͹/, "͹ #{@vid} #{@name}"))
          #        of.print(erbres('skel/head.html'))

          of.flock(File::LOCK_EX)

          of.print "#{@vid} #{@name}<strong>#{update_str}</strong>\n"

          of.print "<p>"
          for i in -1..@date
            datestr =
              if (i == 0)
                "ץ"
              elsif (i == -1)
                ""
              elsif (i == @date)
                "ԥ"
              else
                "#{i}"
              end
            if (i == date)
              of.print "<span class=\"time\">#{datestr}</span> "
            else
              ds = if (i == -1) then 'info' else i.to_s end
              of.print "<a href=\"log_#{@vid}_#{ds}.html\">#{datestr}</a> "
            end
          end
          of.print "</p>\n"

          if (date == -1)
            @vil = self
            of.print(erbres('skel/info.html'))
            a = Array.new
            all_pids.each do |p|
              alive =
                if (p.death == '¸')
                "#{p.survive}֤Ӥ"
                elsif (p.death == '跺')
                  "#{p.survive}ܤ˽跺줿"
                elsif (p.death == '')
                  "#{p.survive}ܤ˽⤵줿"
                elsif (p.death == '')
                  "#{p.survive}ܤ˾Ǥ"
                end
              a.push("#{p.name} (<a href=\"matome.cgi?#{CGI.escape(p.userid)}\">#{p.nick}</a>)#{alive}#{p.skill.name}(#{votes[p].to_i}ɼ)")

            end
            of.print("<br>С:<br>" + a.join('<br>') + "</p>")
            of.print(FOOT)
            next
          end

          File.open("db/vil/#{@vid}_#{date}.html") do |ifile|
            ifile.each do |line|
              line.gsub!(/<a href="\?vid=\d+&date=(\d+)#/) do |m|
                "<a href=\"log_#{@vid}_#{$1}.html#"
              end
              of.print(line)
            end
          end

          if (date != @date && date != -1)
            of.print("<a href=\"log_#{@vid}_#{date+1}.html\"></a>\n")
          end

          print_footer(of, date)
        end

        if (USE_GZIP)
          system("gzip -c #{fname} > #{fname}.gz.html")
          unlinks.push(fname)
        end
        if (File.exists?("db/vil/#{@vid}_#{date}.html"))
          unlinks.push("db/vil/#{@vid}_#{date}.html")
        end
      end

      unlinks.each do |f|
        File.unlink(f)
      end

      vldb = Store.new('db/vil.db')
      vldb.transaction do
        v = vldb['root'].find() do |v|
          v['vid'] == @vid
        end
        nocnt = true if v['level'] == 0
      end

#      if (@end_comment && @end_comment =~ /___/)
#        @counted = nil
#        @end_comment = @end_comment.sub(/___/, '')
#      else
      if (nocnt)
        @counted = nil
      else
        @counted = true

        # create user record
        all_pids.each do |p|
          urv = UserRecord::Vil.new
          urv.name = p.name
          urv.win = (@winner == p.skill.position)
          urv.skill = p.skill.name
          urv.position = p.skill.position
          urv.survive = p.survive
          urv.death = p.death
          urv.comment = ''
          urv.cnt = p.cnt
          urv.pt = p.pt
          urv.hist = p.hist
          urv.mvp_pt = votes[p]
          urv.mvp = (@mvp.to_s.index(p.name) == 0)
          urv.vname = @name
          urv.vid = @vid
          urv.vs = Hash.new
          pt = (@winner == p.skill.position) ? 0.5 : -0.5
          all_pids.each do |d|
            next if (p == d)
            urv.vs[d.userid] = (pt - ((@winner == d.skill.position) ? 0.5 : -0.5)).to_i
          end

          urdb_file = "db/ur/#{CGI.escape(p.userid)}.db"
          urdb = Store.new(urdb_file)
          urdb.transaction do
            if (!urdb.root?('root'))
              urdb['root'] = UserRecord.new(p.userid, p.nick)
            end
            urdb['root'].add_vil(urv)
          end
          http_broadcast(urdb_file)
        end
      end

      vldb = Store.new('db/vil.db')
      vldb.transaction do
        v = vldb['root'].find() do |v|
          v['vid'] == @vid
        end
        v['winner'] = winner
        v['dist'] = dist
        v['date'] = date - 1
        v['mvp'] = mvp
        v['state'] = 4
        v['is_counted'] = is_counted
      end
      http_broadcast('db/vil.db')

      lock.unlock('vil')
    ensure
      lock.unlock('vil')
    end
  end

  def print_footer(o, d, filter = '1')
    o.print(FOOT)

    if (filter != '0')
      o.print "<div class=\"right\" id=\"right\"><form name=\"checks\"><table>\n"
      o.print "<tr><td></td><td>Cnt.</td><td>Pt.</td><td>̾</td></tr>\n"
      o.print "<tr><td><input type=\"checkbox\" name=\"filter0\" onClick=\"changeFilter(0);\" checked></td><td></td><td></td><td>˵Ѽ</td></tr>\n"
      deadfilter = @state == 2 && !all_pids.select {|p| p.sid == 16}.empty?
      all_pids.each do |p|
        if !deadfilter || p.dead == 0
          o.print "<tr><td><input type=\"checkbox\" name=\"filter#{p.pid}\" onClick=\"changeFilter(#{p.pid});\" checked></td><td>#{p.cnt[d].to_i}</td><td>#{p.pt[d].to_i}</td><td>#{p.sname}</td></tr>\n"
        end
      end
      o.print "</table><input type=\"button\" onClick=\"showAll();\" value=\"ɽ\"> <input type=\"button\" onClick=\"hideAll();\" value=\"ɽ\"></form><br>\n"
      o.print %Q(<form name="reg" onSubmit="document.reg.regF[6].checked = true; return changeRegFilter(document.reg.rstr.value);"><input type="radio" name="regF" checked onClick="changeRegFilter('.');"><br><input type="radio" name="regF" onClick="changeRegFilter('(|||.*?)');"> ڡ<br><input type="radio" name="regF" onClick="changeRegFilter('');"> <br><input type="radio" name="regF" onClick="changeRegFilter('');"> <br><input type="radio" name="regF" onClick="changeRegFilter('');"> <br><input type="radio" name="regF" onClick="changeRegFilter('.*?');">ڡ<br><input type="radio" name="regF" onClick="changeRegFilter(document.reg.rstr.value);"> <input name="rstr" size="10">)
      o.print(DISPLAY_FILTER)
    end
    o.print("</div></body></html>")
  end

  def up_mirror
    lefts = @nums.clone
    Skill.skills.each_index do |i|
      if (lefts[i] == 0)
        @players.values.select { |p| p.sid == i }.each do |p|
          p.sid = -1
        end
        next
      end

      ps = @players.values.select { |p| p.sid == i }
      if (ps.size > lefts[i])
        for j in 0...ps.size-lefts[i]
          r = rand(ps.size)
          r = rand(ps.size) while (ps[r].sid == -1)
          ps[r].sid = -1
        end
        lefts[i] = 0
      else
        lefts[i] -= ps.size
      end
    end

    ps = @players.values.select { |p| p.sid == -1 }

    shrink_cnt = 0
    lefts.each do |c|
      shrink_cnt += c
    end
    shrink_cnt -= ps.size

    for i in 0...shrink_cnt
      r = rand(lefts.size).to_i
      r = rand(lefts.size).to_i while (lefts[r] == 0)
      lefts[r] -= 1
      @nums[r] -= 1
    end

    if (ps.size > 0)
      Skill.skills.each_index do |i|
        next if (lefts[i] == 0)

        for j in 0...lefts[i]
          r = rand(ps.size).to_i
          r = rand(ps.size).to_i while (ps[r].sid != -1)
          ps[r].sid = i
        end
      end
    end

    if (@gert && player_p(1).sid != 0)
      ps = pids_s(0)
      p = ps[rand(ps.size).to_i]
      p.sid = player_p(1).sid
      player_p(1).sid = 0
    end

    change_state_sync(2)

    s = announce(MIRROR)
    a = Array.new
    nums.each_index do |i|
      next if nums[i] == 0
      a.push("#{SKILLS[i]}#{nums[i]}̾")
    end
    s += announce(c(SHOW_DIST, a.join('')))
    addlog(s)

    if (@guts)
      s = c(SHOW_WOLFGUTS, wolves.collect { |p| p.nick }.sort.join(''))
      addlog(announce(s))
    end

    if (@gert)
      s = say('say', nil, player_p(1), GERT_FIRST)
      addlog(message(s))
    end

    free = pids_s(6)
    free.each do |p|
      a = Array.new
      free.each do |p2|
        next if (p == p2)
        a.push(p2.name)
      end
      p.msg = c(SHOW_FREES, a.join(''))
    end

    pids_s(9).each do |p|
      p.msg = c(SHOW_WOLFS, wolves.collect { |w| w.name }.join(''))
    end

    blessed = pids_s(10)
    blessed.each_index do |i|
      blessed[i].msg = c(SHOW_SAINT, i+1)
    end
  end

  def up_fortune
    vanish = Array.new

    pids_s(2).each do |p|
      next if (!p.target)

      t = player_p(p.target)
      addlog(extra(p.pid, c(SEERING, p.name, t.name)))
      t.add_hist("#{@date}ܤ줿")

      if (t.dead == 0 && t.sid == 7)
        t.dead = 1
        vanish.push(announce(c(VANISHING, t.name) + '<br>'))
        #addlog(announce(c(VANISHING, t.name)))
        p.msg = p.msg.to_s + c(SEER_RESULT, t.name, 'ʹ') + '<br>'
        t.survive = @date
        t.death = ""
      else
        addresmsg(p, t)
      end
    end

    pids_s(11).each do |p|
      next if (!p.target)

      t = player_p(p.target)
      addlog(extra(p.pid, c(GUTS_SEERING, p.name, t.name)))
      t.add_hist("#{@date}ܤꤤ줿")

      p.msg = p.msg.to_s + c(SEER_RESULT, t.name, t.nick) + '<br>'
    end

    [12, 13].each do |sid|
      pids_s(sid).each do |p|
        next if (!p.target)

        t = player_p(p.target)
        addlog(extra(p.pid, c(SKILL_SEERING, p.name, t.name)))
        t.add_hist("#{@date}ܤꤤ줿")

        p.msg = p.msg.to_s + c(SEER_RESULT, t.name, t.skill.name) + '<br>'
      end
    end

    return vanish
  end

  def up_vote
    s = ''
    votes = Hash.new
    max = 0
    decider = Array.new
    pids.each do |p|
      s += c(VOTING, p.name, player_name(p.vote))
      s += "<br>"
      t = player_p(p.vote)
      votes[t] = votes[t].to_i + 1
      if (p.sid == 15 && p.target)
        decider.push(c(VOTING, '', player_name(p.target))+"<br>")
        t = player_p(p.target)
        votes[t] = votes[t].to_i + 1
      end
      max = votes[t] if (votes[t] > max)
    end
    picked = votes.select { |k, v| v == max }
    result = picked[rand(picked.size).to_i][0]
    if (@anon)
      s = ''
    else
      while (decider.size > 0)
        r = rand(decider.size)
        s += decider[r]
        decider.delete_at(r)
      end
      s += '<br>'
    end
    votes.each_pair do |p, n|
      s += c(ANON_VOTING, n, p.name) + '<br>'
    end

    s += "<br>"
    s += c(EXECUTION, result.name)
    addlog(announce(s))
    result.add_hist("#{@date}ܤ˽跺줿")
    result.survive = @date
    result.death = "跺"
    result.dead = 1

    if result.sid == 16
      result.transform
    end

    pids_s(3).each do |p|
      addresmsg(p, result)
    end
    #
    pids_s(18).each do |p|
      p.msg = p.msg.to_s + c(SEER_RESULT, result.name, result.nick) + '<br>'
    end
    #
    pids_s(19).each do |p|
      p.msg = p.msg.to_s + c(SEER_RESULT, result.name, result.skill.name) + '<br>'
    end
  end

  def up_attack(vanish)
    wolfs = wolves.select { |p| p.target != nil }
    t = nil
    w = nil
    if (wolfs.size > 0)
      w = wolfs[rand(wolfs.size).to_i]
      t = player_p(w.target)

      m = message(say('whisper', nil, w, c(ATTACKING, t.name)))
      addlog(m)
      t.add_hist("#{@date}ܤ˽⤵줿")
      t.survive = @date
      t.death = ""
    end

    pids_s(5).each do |p|
      next if (!p.target)
      g = player_p(p.target)
      addlog(extra(p.pid, c(GUARDING, p.name, g.name)))
      g.add_hist("#{@date}ܤɱҤ줿")
    end

    if (wolfs.size > 0)
      if (t.dead != 0)
        t = nil
      elsif (!(@gert && @date == 1))
        is_gj = false
        pids_s(5).each do |p|
          next if (!p.target)
          g = player_p(p.target)
          if (t == g)
            is_gj = true
            p.msg = p.msg.to_s + c(GJ, g.name) + '<br>'
          end
        end
        t = nil if (is_gj)
      end
      if (t && t.sid == 7)
        t = nil
      end
    end

    killed = nil
    if (t)
      name = t.name
      if w.sid == 16 && (!@gert || t.pid != 1)
        name = w.name
        w.transform(t.pid)
        pids_s(2).each do |seer|
          if (seer.target == w.pid)
            seer.msg = seer.msg.sub(/[^>]+?<br>$/, '')
            addresmsg(seer, w)
          end
        end
        [12, 13].each do |sid|
          pids_s(sid).each do |seer|
            if (seer.target == w.pid)
              seer.msg = seer.msg.sub(/[^>]+?<br>$/, '')
              seer.msg = seer.msg.to_s + c(SEER_RESULT, w.name, w.skill.name) + '<br>'
            end
          end
        end
      end

      killed = announce(c(KILLED, name) + "<br>")
      #addlog(announce(c(KILLED, t.name) + "<br>"))
      t.dead = 1
      wolves.each do |p|
        p.msg = p.msg.to_s + c(KILL, t.name) + '<br>'
      end
    end

    vanish.push(killed) if killed

    if (vanish.empty?)
      addlog(announce(KILLMISS+"<br>"))
    else
      until vanish.empty?
        r = rand(vanish.size)
        addlog(vanish.delete_at(r))
      end
    end
  end

  def up_gameover
    wcnt = wolves.size
    ycnt = pids_s(7).size
#    ccnt = pids_s(8).size
#    ccnt = 0 if ycnt > 0
#    if (wcnt == 0 || wcnt*2 >= (pids.size - ycnt - ccnt))
    if (wcnt == 0 || wcnt*2 >= (pids.size - ycnt))
      pids_s(16).each do |w|
        w.transform
      end

      change_state_sync(3)
      @night = nil

      if (pids_s(7).size > 0)
        if (wcnt == 0)
          addlog(announce(YOKO_WIN_F))
        else
          addlog(announce(YOKO_WIN_W))
        end
        @winner = ''
      elsif (wcnt == 0)
        addlog(announce(FOLK_WIN))
        @winner = '¼'
      else
        addlog(announce(WOLF_WIN))
        @winner = 'ϵ'
      end

      a = Array.new
      all_pids.each do |p|
        alive = if (p.dead == 0)
                  '¸'
                else
                  '˴'
                end
        a.push(c(TRUTH, p.name, "<a href=\"matome.cgi?#{CGI.escape(p.userid)}\">#{p.nick}</a>", alive, p.skill_name))

        if (p.dead == 0)
          p.survive = @date
          p.death = '¸'
        end
      end
      addlog(announce(a.join('<br>')))

      all_pids.each do |p|
        p.dead = 0
        p.msg = ''
      end
    end
  end

  def up_gertdie
    s = announce(GERTDIE)
    addlog(s)
  end

  def up_showlives
    s = c(LIVES, pids.collect { |p| p.name }.join(''), pids.size)
    addlog(announce(s))
  end

  def up_resetcnt
    @say_cnt['say'] = 0
    @say_cnt['think'] = 0
    @say_cnt['groan'] = 0
    @say_cnt['whisper'] = 0
  end

  def up_resetcommit
    pids.each do |p|
      if (p.pid == 1 && @gert)
        p.commit = true
      else
        p.commit = false
      end
    end
  end

  def up_reset
    # reset cmd
    pids.each do |p|
      p.vote = pids[rand(pids.size).to_i].pid
      p.vote = pids[rand(pids.size).to_i].pid while (p.vote == p.pid)
      p.target = nil
    end

    if (@gert && @date == 1)
      wolves.each do |w|
        w.target = 1
      end
    end
  end

  def up_uptime(period = @period)
    now = Time.now.to_i
    if (@update_time && @update_time < now)
      @update_time += period * 60
    else
      @update_time = now + period * 60
    end
  end

  def day_update
    @night = true

    up_uptime(@night_period)

    @date += 1
    addlog('')

    if (@date >= 3)
      up_vote
    end

    up_resetcnt
    up_resetcommit

    up_gameover
  end

  def night_update
    @night = false

    up_uptime

    vanish =
      if (@date >= 2)
        up_fortune
      end

    if (@date >= 2)
      up_attack(vanish)
    end

    up_gameover

    if (@state == 3)
      return
    end

    if (@date == 2)
      up_gertdie
    end

    if (@date != 1)
      up_showlives
    end

    up_reset
    up_resetcommit
  end

  def first_night_in
    @night = true
    up_uptime(@night_period)
    @date += 1
    up_resetcnt
    up_mirror
  end

  def allday_update
    up_uptime

    @date += 1

    # opening
    if (@date == 1)
      up_mirror
    end

    if (@date >= 3)
      up_vote
    end

    vanish =
      if (@date >= 2)
        up_fortune
      end

    if (@date >= 2)
      up_attack(vanish)
    end

    up_resetcnt

    up_gameover

    if (@state == 3)
      return
    end

    if (@date == 2)
      up_gertdie
    end

    if (@date != 1)
      up_showlives
    end

    up_reset
    up_resetcommit
  end

  def update(nocnt)
    if (@state == 4)
      return
    end

    if (@state == 3)
      do_end(nocnt)
      @update_time = Time.now.to_i
      @state = 4
      return
    end

    if (@night_period)
      if (@date == 0)
        allday_update
        @night = false
      elsif (@night)
        night_update
      else
        day_update
      end
    else
      allday_update
    end
  end

  def addlog(msg)
    File.open("db/vil/#{@vid}_#{@date}.html", 'a') do |of|
      of.flock(File::LOCK_EX)
      of.print(msg)
    end
  end

  def all_pt
    s = 0
    all_pids.each do |p|
      s += p.all_pt
    end
    s
  end

  def all_cnt
    s = 0
    all_pids.each do |p|
      s += p.all_cnt
    end
    s
  end

  def is_counted
    counted
  end

  def recnt
    raise ErrorMsg.new('̤') if (!@counted)

    lock = TSSLock.new(COUNTRY_NAME)
    raise ErrorMsg.new('ʤ󤫥åǤƤޤ') if (!lock.lock('vil'))

    begin
      @counted = nil

      all_pids.each do |p|
        urdb = Store.new("db/ur/#{CGI.escape(p.userid)}.db")
        urdb.transaction do
          urdb['root'].del_vil(@vid)
        end
        http_broadcast("db/ur/#{CGI.escape(p.userid)}.db")
      end

      vldb = Store.new('db/vil.db')
      vldb.transaction do
        v = vldb['root'].find() do |v|
          v['vid'] == @vid
        end
        v['is_counted'] = nil
      end

      http_broadcast('db/vil.db')

      lock.unlock('vil')
    ensure
      lock.unlock('vil')
    end
  end

  def say(type, cnt, player, msg, face = nil)
    face = "face#{player.color}" if !face

    name = player.name
    name += " (#{player.nick})" if guts2 || @state == 3

    say_impl(type, cnt, player.pid, name, face, msg)
  end

  def say_action(type, player, msg)
    if type == 'action'
      s = "<div id=\"#{type}#{player.pid}p\"><div class=\"announce\"><div class=\"#{type}\">#{player.name}ϡ#{msg} <span class=\"time\">(#{timestr})</span></div></div></div>"
    else
      s = ''
    end
  end

  def say_ref(m, v1, v2, v3, v4) 
    t = if (v1 == '*')
          'whisper'
        elsif (v1 == '-')
          'think'
        elsif (v1 == '+')
          'groan'
        else
          'say'
        end

    colon = v3
    dt, nm = (colon) ? [v2, v4] : [@date, v2]
    title = ''
    logfile = "db/vil/#{@vid}_#{dt}.html"
    if ((@state == 3 || t == 'say') && FileTest.exist?(logfile))
      File.open(logfile) do |ifile|
        reg = /^<div class="message"><div id="[^"]+"><a name="#{t}#{nm}">[^ ]+ ([^<]+)<\/a> <span class="time">([^<]+)<\/span> <a href="[^"]+">\([^)]+\)<\/a><table border=0 cellpadding=0 cellspacing=0 class="message_box"><tr><td width="40"><img src="img\/[^\.]+.jpg"><\/td><td width="16"><img src="img\/[^\d]+00.jpg"><\/td><td> <div class="mes_[^_]+_body0"> <div class="mes_[^_]+_body1">(.*)$/
        ifile.each do |line|
          if (reg =~ line)
            rn = $1
            rt = $2
            rm = $3
            rm.gsub!(/<a [^>]+>/, '')
            rm.gsub!(/<\/a>/, '')
            rm.gsub!(/<br>/, ' ')
            rm.gsub!(/<div class="loud">/, '')
            rm.gsub!(/<\/div>/, '')
            rm.gsub!(/<\/td>/, '')
            rm.gsub!(/<\/tr>/, '')
            rm.gsub!(/<\/table>/, '')
            rm.sub!(/ +$/, '')
            title = %Q( title="#{rn}: #{rm}")
            break
          end
        end
      end
    end

    if (colon)
      "<a class=\"say\" href=\"?vid=#{@vid}&date=#{dt}##{t}#{nm}\"#{title}>#{m}</a>"
    else
      "<a class=\"say\" href=\"##{t}#{nm}\"#{title}>#{m}</a>"
    end
  end

  def say_impl(type, cnt, pid, name, face, msg)
    mark =
      if (type == 'whisper')
        '*'
      elsif (type == 'think')
        '-'
      elsif (type == 'groan')
        '+'
      else
        ''
      end

    msg.gsub!(/(&gt;&gt;(\d+)(:([*+-]?)(\d+))?|&gt;&gt;([*+-]?)(\d+)(:(\d+))?)/) do |m|
      if ($2)
        say_ref(m, $4, $2, $3, $5)
      else
        say_ref(m, $6, $7, $8, $9)
      end
    end

    s = "<div id=\"#{type}#{pid}p\"><a name=\"#{type}#{cnt}\">#{name}</a> <span class=\"time\">#{timestr}</span> "
    if (cnt) then
      s += "<a href=\"##{type}#{cnt}\">(#{mark}#{cnt})</a>"
    end
    s += "<table border=0 cellpadding=0 cellspacing=0 class=\"message_box\"><tr><td width=\"40\"><img src=\"img/#{face}.jpg\"></td><td width=\"16\"><img src=\"img/#{type}00.jpg\"></td><td> <div class=\"mes_#{type}_body0\"> <div class=\"mes_#{type}_body1\">#{msg}</div> </div> </td></tr></table></div>"
    s
  end

  def onsubmit_str(login_preview)
    prev = @preview
    prev = true if login_preview == '1'
    prev = false if login_preview == '0'
    if (prev)
      'onSubmit="return preview(this);"'
    else
      ''
    end
  end

  def dist
    dist = ''
    nums.each_index do |i|
      for j in 0...nums[i]
        if (i == 0 && j == 0 && gert)
          dist += ''
        else
          dist += Skill.skills[i].sname
        end
      end
    end
    if (guts)
      dist += "/"
    end
    dist
  end

  def player_name(p)
    Player.name(moe, p)
  end

end
